@@ -33,10 +33,8 @@ def test_defaults(loop):
                 assert_can_connect_from_everywhere_4_6(8786, 5.0),  # main port
             ]
 
-        loop.run_sync(f)
-
         with Client('127.0.0.1:%d' % Scheduler.default_port, loop=loop) as c:
-            pass
+            c.sync(f)
 
     with pytest.raises(Exception):
         requests.get('http://127.0.0.1:8787/status/')
@@ -50,13 +48,12 @@ def test_hostport(loop):
         def f():
             yield [
                 # The scheduler's main port can't be contacted from the outside
-                assert_can_connect_locally_4(8978, 2.0),
+                assert_can_connect_locally_4(8978, 5.0),
             ]
 
-        loop.run_sync(f)
-
         with Client('127.0.0.1:8978', loop=loop) as c:
             assert len(c.ncores()) == 0
+            c.sync(f)
 
 
 def test_no_bokeh(loop):
@@ -104,7 +101,7 @@ def test_bokeh_non_standard_ports(loop):
 
     with popen(['dask-scheduler',
                 '--port', '3448',
-                '--bokeh-port', '4832']) as proc:
+                '--dashboard-address', ':4832']) as proc:
         with Client('127.0.0.1:3448', loop=loop) as c:
             pass
 
@@ -229,7 +226,7 @@ def test_scheduler_port_zero(loop):
 def test_bokeh_port_zero(loop):
     pytest.importorskip('bokeh')
     with tmpfile() as fn:
-        with popen(['dask-scheduler', '--bokeh-port', '0']) as proc:
+        with popen(['dask-scheduler', '--dashboard-address', ':0']) as proc:
             count = 0
             while count < 1:
                 line = proc.stderr.readline()
