@@ -1,11 +1,11 @@
 from __future__ import print_function, division, absolute_import
 
-from collections import Iterator
 import datetime
 from functools import partial
 import io
 import socket
 import sys
+from time import sleep
 import traceback
 
 import numpy as np
@@ -14,14 +14,14 @@ from tornado import gen
 from tornado.ioloop import IOLoop
 
 import dask
-from distributed.compatibility import Queue, Empty, isqueue, PY2
+from distributed.compatibility import Queue, Empty, isqueue, PY2, Iterator
 from distributed.metrics import time
 from distributed.utils import (All, sync, is_kernel, ensure_ip, str_graph,
                                truncate_exception, get_traceback, queue_to_iterator,
                                iterator_to_queue, _maybe_complex, read_block, seek_delimiter,
                                funcname, ensure_bytes, open_port, get_ip_interface, nbytes,
                                set_thread_state, thread_state, LoopRunner,
-                               parse_bytes, parse_timedelta)
+                               parse_bytes, parse_timedelta, warn_on_duration)
 from distributed.utils_test import loop, loop_in_thread  # noqa: F401
 from distributed.utils_test import (div, has_ipv6, inc, throws, gen_test,
         captured_logger)
@@ -59,12 +59,6 @@ def test_All(loop):
     loop.run_sync(f)
 
 
-def test_sync(loop_in_thread):
-    loop = loop_in_thread
-    result = sync(loop, inc, 1)
-    assert result == 2
-
-
 def test_sync_error(loop_in_thread):
     loop = loop_in_thread
     try:
@@ -520,16 +514,30 @@ def test_parse_timedelta():
 def test_all_exceptions_logging():
     @gen.coroutine
     def throws():
-        raise Exception('foo')
+        raise Exception('foo1234')
 
     with captured_logger('') as sio:
         try:
             yield All([throws() for _ in range(5)],
-                    quiet_exceptions=Exception)
+                      quiet_exceptions=Exception)
         except Exception:
             pass
 
         import gc; gc.collect()
         yield gen.sleep(0.1)
 
-    assert not sio.getvalue()
+    assert 'foo1234' not in sio.getvalue()
+
+
+def test_warn_on_duration():
+    with pytest.warns(None) as record:
+        with warn_on_duration('10s', 'foo'):
+            pass
+    assert not record
+
+    with pytest.warns(None) as record:
+        with warn_on_duration('1ms', 'foo'):
+            sleep(0.100)
+
+    assert record
+    assert any('foo' in str(rec.message) for rec in record)
